@@ -12,9 +12,13 @@ import numpy as np
 from astropy.wcs import (WCSSUB_SPECTRAL, WCSSUB_LONGITUDE, WCSSUB_LATITUDE)
 from . import wcs_utils
 from astropy import log
+from astropy.io import fits
 from astropy.io.fits import BinTableHDU, Column
 from astropy import units as u
 import itertools
+import re
+from radio_beam import Beam
+
 
 def _fix_spectral(wcs):
     """
@@ -27,7 +31,7 @@ def _fix_spectral(wcs):
 
     types = [a['coordinate_type'] for a in axtypes]
 
-    if wcs.naxis not in (3,4):
+    if wcs.naxis not in (3, 4):
         raise TypeError("The WCS has {0} axes of types {1}".format(len(types),
                                                                    types))
 
@@ -144,6 +148,22 @@ def _orient(array, wcs):
 
     types = [a['coordinate_type'] for a in axtypes]
 
+    n_celestial = types.count('celestial')
+
+    if n_celestial == 0:
+        raise ValueError('No celestial axes found in WCS')
+    elif n_celestial != 2:
+        raise ValueError('WCS should contain 2 celestial dimensions but '
+                         'contains {0}'.format(n_celestial))
+
+    n_spectral = types.count('spectral')
+
+    if n_spectral == 0:
+        raise ValueError('No spectral axes found in WCS')
+    elif n_spectral != 1:
+        raise ValueError('WCS should contain one spectral dimension but '
+                         'contains {0}'.format(n_spectral))
+
     nums = [None if a['coordinate_type'] != 'celestial' else a['number']
             for a in axtypes]
 
@@ -243,6 +263,66 @@ def iterator_strategy(cube, axis=None):
     return 'slice'
 
 
+def try_load_beam(header):
+    '''
+    Try loading a beam from a FITS header.
+    '''
+
+    try:
+        beam = Beam.from_fits_header(header)
+        return beam
+    except Exception as ex:
+        # We don't emit a warning if no beam was found since it's ok for
+        # cubes to not have beams
+        if 'No BMAJ' not in str(ex):
+            warnings.warn("Could not parse beam information from header."
+                          "  Exception was: {0}".format(ex.__repr__()))
+
+def try_load_beams(data):
+    '''
+    Try loading a beam table from a FITS HDU list.
+    '''
+    try:
+        from radio_beam import Beam
+    except ImportError:
+        warnings.warn("radio_beam is not installed. No beam "
+                      "can be created.")
+
+    if isinstance(data, fits.BinTableHDU):
+        if 'BPA' in data.data.names:
+            beam_table = data.data
+            return beam_table
+        else:
+            raise ValueError("No beam table found")
+    elif isinstance(data, fits.HDUList):
+
+        for ihdu, hdu_item in enumerate(data):
+            if isinstance(hdu_item, (fits.PrimaryHDU, fits.ImageHDU)):
+                beam = try_load_beams(hdu_item.header)
+            elif isinstance(hdu_item, fits.BinTableHDU):
+                if 'BPA' in hdu_item.data.names:
+                    beam_table = hdu_item.data
+                    return beam_table
+
+        try:
+            # if there was a beam in a header, but not a beam table
+            return beam
+        except NameError:
+            # if the for loop has completed, we didn't find a beam table
+            raise ValueError("No beam table found")
+    elif isinstance(data, (fits.PrimaryHDU, fits.ImageHDU)):
+        return try_load_beams(data.header)
+    elif isinstance(data, fits.Header):
+        try:
+            beam = Beam.from_fits_header(data)
+            return beam
+        except Exception as ex:
+            warnings.warn("Could not parse beam information from header."
+                          "  Exception was: {0}".format(ex.__repr__()))
+    else:
+        raise ValueError("How did you get here?  This is some sort of error.")
+
+
 def beams_to_bintable(beams):
     """
     Convert a list of beams to a CASA-style BinTableHDU
@@ -262,7 +342,26 @@ def beams_to_bintable(beams):
     bmhdu.header['NPOL'] = len(set([bm.meta['POL'] for bm in beams]))
     return bmhdu
 
-def average_beams(beams, includemask=None):
+
+def beam_props(beams, includemask=None):
+    '''
+    Returns separate quantities for the major, minor, and PA of a list of
+    beams.
+    '''
+    if includemask is None:
+        includemask = itertools.cycle([True])
+
+    major = u.Quantity([bm.major for bm, incl in zip(beams, includemask)
+                        if incl], u.deg)
+    minor = u.Quantity([bm.minor for bm, incl in zip(beams, includemask)
+                        if incl], u.deg)
+    pa = u.Quantity([bm.pa for bm, incl in zip(beams, includemask)
+                     if incl], u.deg)
+
+    return major, minor, pa
+
+
+def average_beams(beams, includemask=None, raise_for_nan=True):
     """
     Average the beam major, minor, and PA attributes.
 
@@ -272,17 +371,51 @@ def average_beams(beams, includemask=None):
     from radio_beam import Beam
     from astropy.stats import circmean
 
-    if includemask is None:
-        includemask = itertools.cycle([True])
+    major, minor, pa = beam_props(beams, includemask)
+
+    if raise_for_nan and (np.any(np.isnan(major)) or np.any(np.isnan(minor)) or
+                          np.any(np.isnan(pa))):
+        raise ValueError("Some beam parameters are NaN.  Try masking out the bad beams.")
 
-    major = u.Quantity([bm.major for bm,incl in zip(beams,includemask) if incl], u.deg)
-    minor = u.Quantity([bm.minor for bm,incl in zip(beams,includemask) if incl], u.deg)
-    pa = u.Quantity([bm.pa for bm,incl in zip(beams,includemask) if incl], u.deg)
     new_beam = Beam(major=major.mean(), minor=minor.mean(),
                     pa=circmean(pa, weights=major/minor))
 
+    if raise_for_nan and np.any(np.isnan(new_beam)):
+        raise ValueError("NaNs after averaging.  This is a bug.")
+
     return new_beam
 
+
+def largest_beam(beams, includemask=None):
+    """
+    Returns the largest beam (by area) in a list of beams.
+    """
+
+    from radio_beam import Beam
+
+    major, minor, pa = beam_props(beams, includemask)
+    largest_idx = (major * minor).argmax()
+    new_beam = Beam(major=major[largest_idx], minor=minor[largest_idx],
+                    pa=pa[largest_idx])
+
+    return new_beam
+
+
+def smallest_beam(beams, includemask=None):
+    """
+    Returns the smallest beam (by area) in a list of beams.
+    """
+
+    from radio_beam import Beam
+
+    major, minor, pa = beam_props(beams, includemask)
+    smallest_idx = (major * minor).argmin()
+    new_beam = Beam(major=major[smallest_idx], minor=minor[smallest_idx],
+                    pa=pa[smallest_idx])
+
+    return new_beam
+
+
 @contextlib.contextmanager
 def _map_context(numcores):
     """
@@ -309,3 +442,32 @@ def _map_context(numcores):
     finally:
         if parallel:
             p.close()
+
+
+def convert_bunit(bunit):
+    '''
+    Convert a BUNIT string to a quantity
+
+    Parameters
+    ----------
+    bunit : str
+        String to convert to an `~astropy.units.Unit`
+
+    Returns
+    -------
+    unit : `~astropy.unit.Unit`
+        Corresponding unit.
+    '''
+
+    # special case: CASA (sometimes) makes non-FITS-compliant jy/beam headers
+    bunit_lower = re.sub("\s", "", bunit.lower())
+    if bunit_lower == 'jy/beam':
+        unit = u.Jy / u.beam
+    else:
+        try:
+            unit = u.Unit(bunit)
+        except ValueError:
+            warnings.warn("Could not parse unit {0}".format(bunit))
+            unit = None
+
+    return unit
