@@ -5,8 +5,10 @@ import abc
 import numpy as np
 from numpy.lib.stride_tricks import as_strided
 
+from astropy.wcs import InconsistentAxisTypesError
+from astropy.io import fits
+
 from . import wcs_utils
-from .lower_dimensional_structures import Projection
 
 
 __all__ = ['MaskBase', 'InvertedMask', 'CompositeMask', 'BooleanArrayMask',
@@ -87,7 +89,10 @@ def view_of_subset(shp1, shp2, view):
         cv_view = [x for ii,x in enumerate([slice(None)]*3)
                    if ii not in dts]
 
-    return cv_view
+    # return type matters
+    # array[[0,0]] = [array[0], array[0]]
+    # array[(0,0)] = array[0,0]
+    return tuple(cv_view)
 
 class MaskBase(object):
 
@@ -134,6 +139,9 @@ class MaskBase(object):
     def _exclude(self, data=None, wcs=None, view=()):
         return ~self._include(data=data, wcs=wcs, view=view)
 
+    def any(self):
+        return np.any(self.exclude())
+
     def _flattened(self, data, wcs=None, view=()):
         """
         Return a flattened array of the included elements of cube
@@ -203,7 +211,8 @@ class MaskBase(object):
         raise NotImplementedError("Slicing not supported by mask class {0}"
                                   .format(self.__class__.__name__))
 
-    def quicklook(self, view, wcs=None, filename=None, use_aplpy=True):
+    def quicklook(self, view, wcs=None, filename=None, use_aplpy=True,
+                  aplpy_kwargs={}):
         '''
         View a 2D slice of the mask, specified by view.
 
@@ -215,13 +224,38 @@ class MaskBase(object):
             WCS object to use in plotting the mask slice.
         filename : str, optional
             Filename of the output image. Enables saving of the plot.
+        use_aplpy : bool, optional
+            Try plotting with the aplpy package
+        aplpy_kwargs : dict, optional
+            kwargs passed to `~aplpy.FITSFigure`.
         '''
 
-        view_twod = self.include(view=view)
+        view_twod = self.include(view=view, wcs=wcs)
 
-        proj = Projection(view_twod, wcs=wcs)
+        if use_aplpy:
 
-        proj.quicklook(filename=filename, use_aplpy=use_aplpy)
+            if wcs is not None:
+                hdu = fits.PrimaryHDU(view_twod.astype(int), wcs.to_header())
+            else:
+                hdu = fits.PrimaryHDU(view_twod.astype(int))
+
+            try:
+                import aplpy
+                FITSFigure = aplpy.FITSFigure(hdu,
+                                              **aplpy_kwargs)
+
+                FITSFigure.show_grayscale()
+                FITSFigure.add_colorbar()
+                if filename is not None:
+                    FITSFigure.save(filename)
+            except (InconsistentAxisTypesError, ImportError):
+                use_aplpy = True
+
+        if not use_aplpy:
+            from matplotlib import pyplot
+            figure = pyplot.imshow(view_twod)
+            if filename is not None:
+                figure.savefig(filename)
 
     def _get_new_wcs(self, unit, velocity_convention=None, rest_value=None):
         """
@@ -424,7 +458,8 @@ class BooleanArrayMask(MaskBase):
     def __getitem__(self, view):
         return BooleanArrayMask(self._mask[view],
                                 wcs_utils.slice_wcs(self._wcs, view,
-                                                    shape=self.shape),
+                                                    shape=self.shape,
+                                                    drop_degenerate=True),
                                 shape=self._mask[view].shape)
 
     def with_spectral_unit(self, unit, velocity_convention=None, rest_value=None):
@@ -505,7 +540,8 @@ class LazyMask(MaskBase):
     def __getitem__(self, view):
         return LazyMask(self._function, data=self._data[view],
                         wcs=wcs_utils.slice_wcs(self._wcs, view,
-                                                shape=self._data.shape))
+                                                shape=self._data.shape,
+                                                drop_degenerate=True))
 
     def with_spectral_unit(self, unit, velocity_convention=None, rest_value=None):
         """
@@ -572,7 +608,7 @@ class LazyComparisonMask(LazyMask):
     def _include(self, data=None, wcs=None, view=()):
         self._validate_wcs(data, wcs)
 
-        if hasattr(self._comparison_value, 'shape'):
+        if hasattr(self._comparison_value, 'shape') and self._comparison_value.shape:
             cv_view = view_of_subset(self._comparison_value.shape,
                                      self._data.shape, view)
 
@@ -584,16 +620,18 @@ class LazyComparisonMask(LazyMask):
                                   self._comparison_value)
 
     def __getitem__(self, view):
-        if hasattr(self._comparison_value, 'shape'):
+        if hasattr(self._comparison_value, 'shape') and self._comparison_value.shape:
             cv_view = view_of_subset(self._comparison_value.shape,
                                      self._data.shape, view)
             return LazyComparisonMask(self._function, data=self._data[view],
                                       comparison_value=self._comparison_value[cv_view],
-                                      wcs=wcs_utils.slice_wcs(self._wcs, view))
+                                      wcs=wcs_utils.slice_wcs(self._wcs, view,
+                                                              drop_degenerate=True))
         else:
             return LazyComparisonMask(self._function, data=self._data[view],
                                       comparison_value=self._comparison_value,
-                                      wcs=wcs_utils.slice_wcs(self._wcs, view))
+                                      wcs=wcs_utils.slice_wcs(self._wcs, view,
+                                                              drop_degenerate=True))
 
     def with_spectral_unit(self, unit, velocity_convention=None, rest_value=None):
         """
