@@ -14,7 +14,7 @@ from .. import (BooleanArrayMask, SpectralCube, LazyMask, LazyComparisonMask,
                 FunctionMask, CompositeMask)
 from ..masks import is_broadcastable_and_smaller, dims_to_skip, view_of_subset
 
-from distutils.version import StrictVersion
+from distutils.version import LooseVersion
 
 
 def test_spectral_cube_mask():
@@ -350,9 +350,9 @@ def test_dims_to_skip(shp1, shp2, dim):
     assert dims_to_skip(shp1, shp2) == dim
 
 @pytest.mark.parametrize(('shp1','shp2', 'inview', 'outview'),
-                         (([5,5],[2,5,5],  [slice(0,1), slice(1,3), slice(2,4),], [slice(1,3), slice(2,4)]),
+                         (([5,5],[2,5,5],  (slice(0,1), slice(1,3), slice(2,4),), (slice(1,3), slice(2,4))),
                           # not a valid broadcast ([5,5],[5,5,2],  [slice(1,3), slice(2,4), slice(0,1),], [slice(1,3), slice(2,4)]),
-                          ([2,5,5],[2,5,5],[slice(0,1), slice(1,3), slice(2,4),], [slice(0,1), slice(1,3), slice(2,4),])))
+                          ([2,5,5],[2,5,5],(slice(0,1), slice(1,3), slice(2,4),), (slice(0,1), slice(1,3), slice(2,4),))))
 def test_view_of_subset(shp1, shp2, inview, outview):
     assert view_of_subset(shp1,shp2,inview) == outview
 
@@ -370,7 +370,7 @@ def test_flat_mask():
     assert np.all(cube.sum(axis=0)[mask_array] == mcube.sum(axis=0)[mask_array])
     assert np.all(np.isnan(mcube.sum(axis=0)[~mask_array]))
 
-@pytest.mark.skipif(StrictVersion(np.__version__) < StrictVersion('1.7'),
+@pytest.mark.skipif(LooseVersion(np.__version__) < LooseVersion('1.7'),
                     reason='Numpy <1.7 does not support multi-slice indexing.')
 def test_flat_mask_spectral():
     cube, data = cube_and_raw('adv.fits')
@@ -420,3 +420,126 @@ def test_1d_mask_amp():
     ampd = cube.mask & Mask
 
     ampd.include()
+
+def test_2dcomparison_mask_1d_index():
+    cube, data = cube_and_raw('adv.fits')
+
+    med = cube.median(axis=0)
+    mask = cube > med
+
+    mcube = cube.with_mask(mask)
+
+    assert all(mask[:,1,1].include() ==
+               mask.include()[:,1,1])
+
+    spec = mcube[:,1,1]
+
+    assert spec.ndim == 1
+
+    assert all(spec.mask.include() == mask.include()[:,1,1])
+
+    assert spec[:-1].mask.include().shape == (3,)
+    assert all(spec[:-1].mask.include() == mask.include()[:-1,1,1])
+
+    assert isinstance(spec[0], u.Quantity)
+
+    spec = mcube[:-1,1,1]
+
+    assert spec.ndim == 1
+    assert hasattr(spec, '_fill_value')
+
+    assert all(spec.mask.include() == mask.include()[:-1,1,1])
+
+    assert spec[:-1].mask.include().shape == (2,)
+    assert all(spec[:-1].mask.include() == mask.include()[:-2,1,1])
+
+    assert isinstance(spec[0], u.Quantity)
+
+def test_1dcomparison_mask_1d_index():
+    cube, data = cube_and_raw('adv.fits')
+
+    med = cube.median()
+    mask = cube > med
+
+    mcube = cube.with_mask(mask)
+
+    assert all(mask[:,1,1].include() ==
+               mask.include()[:,1,1])
+
+    spec = mcube[:,1,1]
+
+    assert spec.ndim == 1
+
+    assert all(spec.mask.include() == [True,False,False,True])
+
+    assert spec[:-1].mask.include().shape == (3,)
+    assert all(spec[:-1].mask.include() == [True,False,False])
+
+    assert isinstance(spec[0], u.Quantity)
+
+def test_1dmask_indexing():
+    cube, data = cube_and_raw('adv.fits')
+
+    med = cube.median()
+    mask = cube > med
+
+    mcube = cube.with_mask(mask)
+
+    assert all(mask[:,1,1].include() ==
+               mask.include()[:,1,1])
+
+    spec = mcube[:,1,1]
+
+    badvals = np.array([False,True,True,False], dtype='bool')
+    assert np.all(np.isnan(spec[badvals]))
+    assert not np.any(np.isnan(spec[~badvals]))
+
+def test_numpy_ma_tools():
+    """
+    check that np.ma.core.is_masked works
+    """
+
+    cube, data = cube_and_raw('adv.fits')
+
+    med = cube.median()
+    mask = cube > med
+
+    mcube = cube.with_mask(mask)
+
+    assert np.ma.core.is_masked(mcube)
+    assert np.ma.core.getmask(mcube) is not None
+    assert np.ma.core.is_masked(mcube[:,0,0])
+    assert np.ma.core.getmask(mcube[:,0,0]) is not None
+
+@pytest.mark.xfail
+def test_numpy_ma_tools_2d():
+    """ This depends on 2D objects keeping masks, which depends on #395.
+    so, TODO: un-xfail this """
+
+    cube, data = cube_and_raw('adv.fits')
+
+    med = cube.median()
+    mask = cube > med
+
+    mcube = cube.with_mask(mask)
+
+    assert np.ma.core.is_masked(mcube[0,:,:])
+    assert np.ma.core.getmask(mcube[0,:,:]) is not None
+
+
+def test_filled():
+    """ test that 'filled' works """
+
+    cube, data = cube_and_raw('adv.fits')
+
+    med = cube.median()
+    mask = cube > med
+
+    mcube = cube.with_mask(mask)
+    assert np.isnan(mcube._fill_value)
+
+    filled = mcube.filled(np.nan)
+    filled_ = mcube.filled()
+    assert_allclose(filled, filled_)
+
+    assert (np.isnan(filled) == mcube.mask.exclude()).all()
