@@ -4,18 +4,18 @@ import operator
 import itertools
 import warnings
 import mmap
-from distutils.version import StrictVersion, LooseVersion
-
-# needed to test for warnings later
-warnings.simplefilter('always', UserWarning)
+from distutils.version import LooseVersion
 
 import pytest
 
+import astropy
 from astropy.io import fits
 from astropy import units as u
 from astropy.wcs import WCS
 from astropy.wcs import _wcs
 from astropy.tests.helper import assert_quantity_allclose
+from astropy.extern import six
+from astropy.convolution import Gaussian2DKernel, Tophat2DKernel
 import numpy as np
 
 from .. import (SpectralCube, VaryingResolutionSpectralCube, BooleanArrayMask,
@@ -23,10 +23,19 @@ from .. import (SpectralCube, VaryingResolutionSpectralCube, BooleanArrayMask,
 from ..spectral_cube import OneDSpectrum, Projection
 from ..np_compat import allbadtonan
 from .. import spectral_axis
+from .. import base_class
 
 from . import path
 from .helpers import assert_allclose, assert_array_equal
 
+# needed to test for warnings later
+warnings.simplefilter('always', UserWarning)
+
+try:
+    import scipy.ndimage
+    SCIPYOK = True
+except ImportError:
+    SCIPYOK = False
 
 try:
     import yt
@@ -42,12 +51,7 @@ try:
 except ImportError:
     BOTTLENECK_INSTALLED = False
 
-try:
-    from radio_beam import Beam
-    RADIO_BEAM_INSTALLED = True
-except ImportError:
-    RADIO_BEAM_INSTALLED = False
-
+from radio_beam import Beam
 
 NUMPY_LT_19 = LooseVersion(np.__version__) < LooseVersion('1.9.0')
 
@@ -124,12 +128,12 @@ translist = [('advs', [0, 1, 2, 3]),
              ('vda', [0, 2, 1]),
              ('adv', [0, 1, 2]),
              ]
-if RADIO_BEAM_INSTALLED:
-    translist.append(('vda_beams', [0, 2, 1]))
+
+translist_vrsc = [('vda_beams', [0, 2, 1])]
 
 class TestSpectralCube(object):
 
-    @pytest.mark.parametrize(('name', 'trans'), translist)
+    @pytest.mark.parametrize(('name', 'trans'), translist + translist_vrsc)
     def test_consistent_transposition(self, name, trans):
         """data() should return velocity axis first, then world 1, then world 0"""
         c, d = cube_and_raw(name + '.fits')
@@ -288,6 +292,54 @@ class TestSpectralCube(object):
         #assert_allclose(c2[1,:,1].value, expected[1,:,1])
         assert_allclose(c2[:,1,1].value, expected[:,1,1])
 
+    @pytest.mark.parametrize(('name', 'trans'), translist_vrsc)
+    def test_getitem_vrsc(self, name, trans):
+        c, d = cube_and_raw(name + '.fits')
+
+        expected = np.squeeze(d.transpose(trans))
+
+        # No pv slices for VRSC.
+
+        assert_allclose(c[0,:,:].value, expected[0,:,:])
+
+        # Not implemented:
+        #assert_allclose(c[0,0,:].value, expected[0,0,:])
+        #assert_allclose(c[0,:,0].value, expected[0,:,0])
+        assert_allclose(c[:,0,0].value, expected[:,0,0])
+
+        assert_allclose(c[1,:,:].value, expected[1,:,:])
+
+        # Not implemented:
+        #assert_allclose(c[1,1,:].value, expected[1,1,:])
+        #assert_allclose(c[1,:,1].value, expected[1,:,1])
+        assert_allclose(c[:,1,1].value, expected[:,1,1])
+
+        c2 = c.with_spectral_unit(u.km/u.s, velocity_convention='radio')
+
+        assert_allclose(c2[0,:,:].value, expected[0,:,:])
+
+        # Not implemented:
+        #assert_allclose(c2[0,0,:].value, expected[0,0,:])
+        #assert_allclose(c2[0,:,0].value, expected[0,:,0])
+        assert_allclose(c2[:,0,0].value, expected[:,0,0])
+
+        assert_allclose(c2[1,:,:].value, expected[1,:,:])
+
+        # Not implemented:
+        #assert_allclose(c2[1,1,:].value, expected[1,1,:])
+        #assert_allclose(c2[1,:,1].value, expected[1,:,1])
+        assert_allclose(c2[:,1,1].value, expected[:,1,1])
+
+        # @pytest.mark.xfail(raises=AttributeError)
+        @pytest.mark.parametrize(('name', 'trans'), translist_vrsc)
+        def test_getitem_vrsc(self, name, trans):
+            c, d = cube_and_raw(name + '.fits')
+
+            expected = np.squeeze(d.transpose(trans))
+
+            assert_allclose(c[:,:,0].value, expected[:,:,0])
+
+
 class TestArithmetic(object):
 
     def setup_method(self, method):
@@ -468,12 +520,12 @@ class TestNumpyMethods(BaseTest):
         scmed = self.c.apply_numpy_function(np.median, axis=0)
         # this checks whether numpy <=1.9.3 has a bug?
         # as far as I can tell, np==1.9.3 no longer has this bug/feature
-        #if StrictVersion(np.__version__) <= StrictVersion('1.9.3'):
+        #if LooseVersion(np.__version__) <= LooseVersion('1.9.3'):
         #    # print statements added so we get more info in the travis builds
-        #    print("Numpy version is: {0}".format(StrictVersion(np.__version__)))
+        #    print("Numpy version is: {0}".format(LooseVersion(np.__version__)))
         #    assert np.count_nonzero(np.isnan(scmed)) == 5
         #else:
-        #    print("Numpy version is: {0}".format(StrictVersion(np.__version__)))
+        #    print("Numpy version is: {0}".format(LooseVersion(np.__version__)))
         assert np.count_nonzero(np.isnan(scmed)) == 6
 
         scmed = self.c.apply_numpy_function(np.nanmedian, axis=0)
@@ -667,7 +719,7 @@ def test_read_write_rountrip(tmpdir):
     assert cube.shape == cube.shape
     assert_allclose(cube._data, cube2._data)
     if (((hasattr(_wcs, '__version__')
-          and StrictVersion(_wcs.__version__) < StrictVersion('5.9'))
+          and LooseVersion(_wcs.__version__) < LooseVersion('5.9'))
          or not hasattr(_wcs, '__version__'))):
         # see https://github.com/astropy/astropy/pull/3992 for reasons:
         # we should upgrade this for 5.10 when the absolute accuracy is
@@ -857,6 +909,18 @@ def test_slice_wcs(view, naxis):
     sl = cube[view]
     assert sl.wcs.naxis == naxis
 
+def test_spectral_slice_preserve_units():
+    cube, data = cube_and_raw('advs.fits')
+    cube = cube.with_spectral_unit(u.km/u.s)
+
+    sl = cube[:,0,0]
+
+    assert cube._spectral_unit == u.km/u.s
+    assert sl._spectral_unit == u.km/u.s
+
+    assert cube.spectral_axis.unit == u.km/u.s
+    assert sl.spectral_axis.unit == u.km/u.s
+
 def test_header_units_consistent():
 
     cube, data = cube_and_raw('advs.fits')
@@ -964,6 +1028,19 @@ def test_twod_numpy(func, how, axis):
     np.testing.assert_equal(proj.value, dproj)
     assert cube.unit == proj.unit
 
+def test_preserves_header_values():
+    # Check that the non-WCS header parameters are preserved during projection
+
+    cube, data = cube_and_raw('advs.fits')
+    cube._meta['BUNIT'] = 'K'
+    cube._unit = u.K
+    cube._header['OBJECT'] = 'TestName'
+
+    proj = cube.sum(axis=0, how='auto')
+    assert isinstance(proj, Projection)
+    assert proj.header['OBJECT'] == 'TestName'
+    assert proj.hdu.header['OBJECT'] == 'TestName'
+
 @pytest.mark.parametrize('func',('sum','std','max','min','mean'))
 def test_oned_numpy(func):
     # Check that a numpy function returns an appropriate spectrum
@@ -993,7 +1070,7 @@ def test_oned_slice():
     assert cube.unit == spec.unit
     assert spec.header['BUNIT'] == cube.header['BUNIT']
 
-@pytest.mark.skipif('not RADIO_BEAM_INSTALLED')
+
 def test_oned_slice_beams():
     # Check that a slice returns an appropriate spectrum
 
@@ -1026,7 +1103,7 @@ def test_oned_collapse():
     assert cube.unit == spec.unit
     assert spec.header['BUNIT'] == cube.header['BUNIT']
 
-@pytest.mark.skipif('not RADIO_BEAM_INSTALLED')
+
 def test_oned_collapse_beams():
     # Check that an operation along the spatial dims returns an appropriate
     # spectrum
@@ -1058,7 +1135,7 @@ def test_preserve_bunit():
     assert cube.unit == u.Jy
     assert cube.header['BUNIT'] == 'Jy'
 
-@pytest.mark.skipif('not RADIO_BEAM_INSTALLED')
+
 def test_preserve_beam():
 
     cube, data = cube_and_raw('advs.fits')
@@ -1067,7 +1144,67 @@ def test_preserve_beam():
 
     assert cube.beam == beam
 
-@pytest.mark.skipif('not RADIO_BEAM_INSTALLED')
+
+def test_beam_attach_to_header():
+
+    cube, data = cube_and_raw('adv.fits')
+
+    header = cube._header.copy()
+    del header["BMAJ"], header["BMIN"], header["BPA"]
+
+    newcube = SpectralCube(data=data, wcs=cube.wcs, header=header,
+                           beam=cube.beam)
+
+    assert cube.header["BMAJ"] == newcube.header["BMAJ"]
+    assert cube.header["BMIN"] == newcube.header["BMIN"]
+    assert cube.header["BPA"] == newcube.header["BPA"]
+
+    # Should be in meta too
+    assert newcube.meta['beam'] == cube.beam
+
+
+def test_beam_custom():
+
+    cube, data = cube_and_raw('adv.fits')
+
+    header = cube._header.copy()
+    beam = Beam.from_fits_header(header)
+    del header["BMAJ"], header["BMIN"], header["BPA"]
+
+    newcube = SpectralCube(data=data, wcs=cube.wcs, header=header)
+
+    # newcube should now not have a beam
+    assert not hasattr(newcube, "beam")
+
+    # Attach the beam
+    newcube = newcube.with_beam(beam=beam)
+
+    assert newcube.beam == cube.beam
+
+    # Header should be updated
+    assert cube.header["BMAJ"] == newcube.header["BMAJ"]
+    assert cube.header["BMIN"] == newcube.header["BMIN"]
+    assert cube.header["BPA"] == newcube.header["BPA"]
+
+    # Should be in meta too
+    assert newcube.meta['beam'] == cube.beam
+
+    # Try changing the beam properties
+    newbeam = Beam(beam.major * 2)
+
+    newcube2 = newcube.with_beam(beam=newbeam)
+
+    assert newcube2.beam == newbeam
+
+    # Header should be updated
+    assert newcube2.header["BMAJ"] == newbeam.major.value
+    assert newcube2.header["BMIN"] == newbeam.minor.value
+    assert newcube2.header["BPA"] == newbeam.pa.value
+
+    # Should be in meta too
+    assert newcube2.meta['beam'] == newbeam
+
+
 def test_multibeam_slice():
 
     cube, data = cube_and_raw('vda_beams.fits')
@@ -1086,15 +1223,38 @@ def test_multibeam_slice():
     np.testing.assert_almost_equal(flatslice.header['BMAJ'],
                                    (0.1/3600.))
 
+def test_basic_unit_conversion():
+
+    cube, data = cube_and_raw('advs.fits')
+    assert cube.unit == u.K
+
+    mKcube = cube.to(u.mK)
+
+    np.testing.assert_almost_equal(mKcube.filled_data[:].value,
+                                   (cube.filled_data[:].value *
+                                    1e3))
+
+
+def test_basic_unit_conversion_beams():
+    cube, data = cube_and_raw('vda_beams.fits')
+    cube._unit = u.K # want beams, but we want to force the unit to be something non-beamy
+    cube._meta['BUNIT'] = 'K'
+
+    assert cube.unit == u.K
+
+    mKcube = cube.to(u.mK)
+
+    np.testing.assert_almost_equal(mKcube.filled_data[:].value,
+                                   (cube.filled_data[:].value *
+                                    1e3))
+
+
 
-@pytest.mark.skipif('not RADIO_BEAM_INSTALLED')
 def test_beam_jtok_array():
 
     cube, data = cube_and_raw('advs.fits')
-    # technically this should be jy/beam, but astropy's equivalency doesn't
-    # handle this yet
-    cube._meta['BUNIT'] = 'Jy'
-    cube._unit = u.Jy
+    cube._meta['BUNIT'] = 'Jy / beam'
+    cube._unit = u.Jy/u.beam
 
     equiv = cube.beam.jtok_equiv(cube.with_spectral_unit(u.GHz).spectral_axis)
     jtok = cube.beam.jtok(cube.with_spectral_unit(u.GHz).spectral_axis)
@@ -1104,7 +1264,36 @@ def test_beam_jtok_array():
                                    (cube.filled_data[:].value *
                                     jtok[:,None,None]).value)
 
-@pytest.mark.skipif('not RADIO_BEAM_INSTALLED')
+    # test that the beam equivalencies are correctly automatically defined
+    Kcube = cube.to(u.K)
+    np.testing.assert_almost_equal(Kcube.filled_data[:].value,
+                                   (cube.filled_data[:].value *
+                                    jtok[:,None,None]).value)
+
+
+def test_multibeam_jtok_array():
+
+    cube, data = cube_and_raw('vda_beams.fits')
+    assert cube.meta['BUNIT'].strip() == 'Jy / beam'
+    assert cube.unit.is_equivalent(u.Jy/u.beam)
+
+    #equiv = [bm.jtok_equiv(frq) for bm, frq in zip(cube.beams, cube.with_spectral_unit(u.GHz).spectral_axis)]
+    jtok = u.Quantity([bm.jtok(frq) for bm, frq in zip(cube.beams, cube.with_spectral_unit(u.GHz).spectral_axis)])
+
+    # don't try this, it's nonsense for the multibeam case
+    # Kcube = cube.to(u.K, equivalencies=equiv)
+    # np.testing.assert_almost_equal(Kcube.filled_data[:].value,
+    #                                (cube.filled_data[:].value *
+    #                                 jtok[:,None,None]).value)
+
+    # test that the beam equivalencies are correctly automatically defined
+    Kcube = cube.to(u.K)
+    np.testing.assert_almost_equal(Kcube.filled_data[:].value,
+                                   (cube.filled_data[:].value *
+                                    jtok[:,None,None]).value)
+
+
+
 def test_beam_jtok():
     # regression test for an error introduced when the previous test was solved
     # (the "is this an array?" test used len(x) where x could be scalar)
@@ -1123,7 +1312,7 @@ def test_beam_jtok():
                                    (cube.filled_data[:].value *
                                     jtok).value)
 
-@pytest.mark.skipif('not RADIO_BEAM_INSTALLED')
+
 def test_varyres_moment():
     cube, data = cube_and_raw('vda_beams.fits')
 
@@ -1139,7 +1328,7 @@ def test_varyres_moment():
     assert "Arithmetic beam averaging is being performed" in str(wrn[-1].message)
     assert_quantity_allclose(m0.meta['beam'].major, 0.25*u.arcsec)
 
-@pytest.mark.skipif('not RADIO_BEAM_INSTALLED')
+
 def test_append_beam_to_hdr():
 
     cube, data = cube_and_raw('advs.fits')
@@ -1163,34 +1352,31 @@ def test_jybeam_upper():
 
     cube, data = cube_and_raw('vda_JYBEAM_upper.fits')
 
-    assert cube.unit == u.Jy
-    if RADIO_BEAM_INSTALLED:
-        assert hasattr(cube, 'beam')
-        np.testing.assert_almost_equal(cube.beam.sr.value,
-                                       (((1*u.arcsec/np.sqrt(8*np.log(2)))**2).to(u.sr)*2*np.pi).value)
+    assert cube.unit == u.Jy/u.beam
+    assert hasattr(cube, 'beam')
+    np.testing.assert_almost_equal(cube.beam.sr.value,
+                                   (((1*u.arcsec/np.sqrt(8*np.log(2)))**2).to(u.sr)*2*np.pi).value)
 
 def test_jybeam_lower():
 
     cube, data = cube_and_raw('vda_Jybeam_lower.fits')
 
-    assert cube.unit == u.Jy
-    if RADIO_BEAM_INSTALLED:
-        assert hasattr(cube, 'beam')
-        np.testing.assert_almost_equal(cube.beam.sr.value,
-                                       (((1*u.arcsec/np.sqrt(8*np.log(2)))**2).to(u.sr)*2*np.pi).value)
+    assert cube.unit == u.Jy/u.beam
+    assert hasattr(cube, 'beam')
+    np.testing.assert_almost_equal(cube.beam.sr.value,
+                                   (((1*u.arcsec/np.sqrt(8*np.log(2)))**2).to(u.sr)*2*np.pi).value)
 
 # Regression test for #257 (https://github.com/radio-astro-tools/spectral-cube/pull/257)
 def test_jybeam_whitespace():
 
     cube, data = cube_and_raw('vda_Jybeam_whitespace.fits')
 
-    assert cube.unit == u.Jy
-    if RADIO_BEAM_INSTALLED:
-        assert hasattr(cube, 'beam')
-        np.testing.assert_almost_equal(cube.beam.sr.value,
-                                       (((1*u.arcsec/np.sqrt(8*np.log(2)))**2).to(u.sr)*2*np.pi).value)
+    assert cube.unit == u.Jy/u.beam
+    assert hasattr(cube, 'beam')
+    np.testing.assert_almost_equal(cube.beam.sr.value,
+                                   (((1*u.arcsec/np.sqrt(8*np.log(2)))**2).to(u.sr)*2*np.pi).value)
+
 
-@pytest.mark.skipif('not RADIO_BEAM_INSTALLED')
 def test_beam_proj_meta():
 
     cube, data = cube_and_raw('advs.fits')
@@ -1251,3 +1437,164 @@ def test_pix_sign():
     assert s>0
     assert y>0
     assert x>0
+
+
+def test_varyres_moment_logic_issue364():
+    """ regression test for issue364 """
+    cube, data = cube_and_raw('vda_beams.fits')
+
+    assert isinstance(cube, VaryingResolutionSpectralCube)
+
+    # the beams are very different, but for this test we don't care
+    cube.beam_threshold = 1.0
+
+    with warnings.catch_warnings(record=True) as wrn:
+        warnings.simplefilter('default')
+        # note that cube.moment(order=0) is different from cube.moment0()
+        # because cube.moment0() calls cube.moment(order=0, axis=(whatever)),
+        # but cube.moment doesn't necessarily have to receive the axis kwarg
+        m0 = cube.moment(order=0)
+
+    if six.PY2:
+        # sad face, tests do not work
+        pass
+    else:
+        assert "Arithmetic beam averaging is being performed" in str(wrn[-1].message)
+    assert_quantity_allclose(m0.meta['beam'].major, 0.25*u.arcsec)
+
+
+def test_mask_bad_beams():
+    cube, data = cube_and_raw('vda_beams.fits')
+
+    # middle two beams have same area
+    masked_cube = cube.mask_out_bad_beams(0.01,
+                                          reference_beam=Beam(0.3*u.arcsec,
+                                                              0.2*u.arcsec,
+                                                              60*u.deg))
+
+    assert np.all(masked_cube.mask.include()[:,0,0] == [False,False,True,False])
+
+    mean = masked_cube.mean(axis=0)
+    assert np.all(mean == cube[2,:,:])
+
+
+    masked_cube2 = cube.mask_out_bad_beams(0.5,)
+
+    mean2 = masked_cube2.mean(axis=0)
+    assert np.all(mean2 == (cube[2,:,:]+cube[1,:,:])/2)
+
+def test_mad_std():
+    cube, data = cube_and_raw('adv.fits')
+
+    if int(astropy.__version__[0]) < 2:
+        with pytest.raises(NotImplementedError) as exc:
+            cube.mad_std()
+
+    else:
+        # mad_std run manually on data
+        result = np.array([[0.15509701,  0.45763670],
+                           [0.55907956,  0.42932451],
+                           [0.48819454,  0.25499305]])
+
+        np.testing.assert_almost_equal(cube.mad_std(axis=0).value, result)
+
+        mcube = cube.with_mask(cube < 0.98*u.K)
+
+        result2 = np.array([[0.15509701,  0.45763670],
+                            [0.55907956,  0.23835865],
+                            [0.48819454,  0.25499305]])
+
+        np.testing.assert_almost_equal(mcube.mad_std(axis=0).value, result2)
+
+def test_caching():
+
+    cube, data = cube_and_raw('adv.fits')
+
+    assert len(cube._cache) == 0
+
+    worldextrema = cube.world_extrema
+
+    assert len(cube._cache) == 1
+
+    # see https://stackoverflow.com/questions/46181936/access-a-parent-class-property-getter-from-the-child-class
+    world_extrema_function = base_class.SpatialCoordMixinClass.world_extrema.fget.wrapped_function
+
+    assert cube.world_extrema is cube._cache[(world_extrema_function, ())]
+    assert worldextrema == cube.world_extrema
+
+def test_spatial_smooth_g2d():
+    cube, data = cube_and_raw('adv.fits')
+
+    #
+    # Guassian 2D smoothing test
+    #
+    g2d = Gaussian2DKernel(3)
+    cube_g2d = cube.spatial_smooth(g2d)
+
+    # Check first slice
+    result0 = np.array([[ 0.06653894,  0.06598313],
+                        [ 0.07206352,  0.07151016],
+                        [ 0.0702898 ,  0.0697944 ]])
+
+    np.testing.assert_almost_equal(cube_g2d[0].value, result0)
+
+    # Check third slice
+    result2 = np.array([[ 0.04217102,  0.04183251],
+                        [ 0.04470876,  0.04438826],
+                        [ 0.04269588,  0.04242956]])
+
+    np.testing.assert_almost_equal(cube_g2d[2].value, result2)
+
+def test_spatial_smooth_t2d():
+    cube, data = cube_and_raw('adv.fits')
+
+    #
+    # Tophat 2D smoothing test
+    #
+    t2d = Tophat2DKernel(3)
+    cube_t2d = cube.spatial_smooth(t2d)
+
+    # Check first slice
+    result0 = np.array([[ 0.14864167,  0.14864167],
+                        [ 0.14864167,  0.14864167],
+                        [ 0.14864167,  0.14864167]])
+
+    np.testing.assert_almost_equal(cube_t2d[0].value, result0)
+
+    # Check third slice
+    result2 = np.array([[ 0.09203958,  0.09203958],
+                        [ 0.09203958,  0.09203958],
+                        [ 0.09203958,  0.09203958]])
+
+    np.testing.assert_almost_equal(cube_t2d[2].value, result2)
+
+@pytest.mark.skipif('not SCIPYOK')
+def test_spatial_smooth_median():
+    cube, data = cube_and_raw('adv.fits')
+
+    cube_median = cube.spatial_smooth_median(3)
+
+    # Check first slice
+    result0 = np.array([[ 0.54671028,  0.54671028],
+                        [ 0.89482735,  0.77513282],
+                        [ 0.93949894,  0.89482735]])
+
+    np.testing.assert_almost_equal(cube_median[0].value, result0)
+
+    # Check third slice
+    result2 = np.array([[ 0.38867729,  0.35675333],
+                        [ 0.38867729,  0.35675333],
+                        [ 0.35675333,  0.54269608]])
+
+    np.testing.assert_almost_equal(cube_median[2].value, result2)
+
+@pytest.mark.skipif('not SCIPYOK')
+def test_spectral_smooth_median():
+    cube, data = cube_and_raw('adv.fits')
+
+    cube_spectral_median = cube.spectral_smooth_median(3)
+
+    # Check first slice
+    result = np.array([ 0.77513282,  0.35675333,  0.35675333,  0.98688694])
+
+    np.testing.assert_almost_equal(cube_spectral_median[:,1,1].value, result)
\ No newline at end of file
